#
# djcommon.mak: make rules for Watt-32 binaries (djgpp2)
#
# Note: This file requires GNU Make 3.75 or later
#       This file requires a non-Unix shell.
#
# by Gisle Vanem 1997
#

.SUFFIXES: .l .y .exe
.PHONY:    check_src check_exe

#
# Override any Unix-like SHELL set in environment or djgpp.env
# At-least "make depend" needs a DOS-style shell. Specifically
# the "sed" command doesn't work with `SHELL=sh' or `bash'.
#
# MAKESHELL = command.com  # doesn't have the wanted effect
# SHELL = $(COMSPEC)       # doesn't work

ifeq ($(SRC),)
  ifeq ($(CC_SRC),)
check_src:
	@echo \"SRC\" or \"CC_SRC\" not defined.
	@echo Define \"SRC = C-files\".. or \"CC_SRC = CC-files\".. before including DJCOMMON.MAK
  endif
endif

ifeq ($(DJGPP_EXE),)
check_exe:
	@echo \"DJGPP_EXE\" not defined. Define \"DJGPP_EXE = file.exe\".. before including DJCOMMON.MAK
endif

OBJS    += $(SRC:.c=.o)
CC_OBJS += $(CC_SRC:.cc=.o)

WATT32_ROOT = $(subst \,/,$(WATT_ROOT))
WATTLIB     = $(WATT32_ROOT)/lib/libwatt.a
MAP_FILE    = $(DJGPP_EXE:.exe=.map)

CC      = gcc
CFLAGS += -g -gcoff -I$(WATT32_ROOT)/inc -Wall -DWATT32

#
# Define 'NO_OPTIMIZE = 1' to ease debugging
#
ifneq ($(NO_OPTIMIZE),1)
  CFLAGS += -O2
endif

#
# Define 'STRIP_EXE = 1' if you like a smaller .exe-file (DJGPP_EXE)
# Define 'PACK_EXE = 1' to also compress using UPX .exe-packer
#
ifeq ($(STRIP_EXE),1)
  LFLAGS += -s
endif

ifeq ($(PACK_EXE),1)
  LFLAGS += -s
endif

#
# Define 'MAKE_MAP = 1' if you like a .map-file
#
ifeq ($(MAKE_MAP),1)
  LINK = redir -e $(MAP_FILE) $(CC) -Wl,--print-map,--sort-common,--cref
else
  LINK = $(CC)
endif

#
# Define 'USE_YAMD = 1' to include a malloc debugged version
#
ifeq ($(USE_YAMD),1)
  CFLAGS += -DYAMD_VERSION=\"0.32\"
  LFLAGS += -Wl,--wrap,malloc,--wrap,realloc,--wrap,free
  SRC    += ../yamd.c
endif


all: $(PREREQUISITES) $(DJGPP_EXE)

#
# Define 'DPMI_STUB = 1' to make a DPMI-host contained .exe-file
#
$(DJGPP_EXE): $(OBJS) $(CC_OBJS) $(WATTLIB)
ifeq ($(DPMI_STUB),1)
	$(LINK) $^ -o $* $(strip $(LFLAGS))
	@copy /b $(subst /,\,$(DJDIR))\bin\cwsdstub.exe + $* $@
	@del $*
else
	$(LINK) $^ -o $@ $(strip $(LFLAGS))
endif
ifeq ($(PACK_EXE),1)
	@upx -9 $@
endif

%.o: %.c
	$(CC) -c $(strip $(CFLAGS)) -o $@ $<

%.o: %.cc
	$(CC) -c $(strip $(CFLAGS)) -fno-rtti -fno-exceptions -o $@ $<


clean:
	rm -f $(OBJS) $(CC_OBJS) Makefile.bak $(DJGPP_EXE:.exe=.map)

vclean scrub dist-clean:  clean
	rm -f $(DJGPP_EXE) $(GENERATED)

#
# The following hack is to bypass a small bug in gcc 2.95.x;
# It includes some system headers in dependency output.
#
MM_SPECS = $(subst \,/,$(TMPDIR))/specs

$(MM_SPECS)::
	@echo "*cpp: %(cpp_cpu) %{posix:-D_POSIX_SOURCE} -remap" > $(MM_SPECS)

depend: $(MM_SPECS)
	@echo Generating dependencies..
	- @copy Makefile.dj Makefile.bak
	sed -e "/^# DO NOT DELETE THIS LINE/,$$d" < Makefile.bak > Makefile.dj
	echo "# DO NOT DELETE THIS LINE"                        >> Makefile.dj
	$(CC) -specs=$(MM_SPECS) -MM $(CFLAGS) $(SRC) $(CC_SRC) >> Makefile.dj
	rm -f $(MM_SPECS)
