

#include "rsync.h"

#if defined(__MSDOS__)

static void fatal (const char *fmt, ...)
{
  va_list args;
  va_start(args, fmt);
  vfprintf (stderr, fmt, args);
  exit (-1);
}

int piped_child (char **command, int *f_in, int *f_out)
{
  struct hostent    *he;
  struct servent    *se;
  struct sockaddr_in sa;
  char   *p, buf[2048];
  int    i, sock;

  if (strcmp(command[0],"rsh"))
     fatal ("piped_child: only \"rsh\" supported\n");

  printf ("piped_child():\n");
  for (i = 0; command[i]; i++)
      printf ("  command[%d] = `%s'\n", i, command[i]);

  sock = socket (AF_INET, SOCK_STREAM, 0);
  if (sock < 0)
     fatal ("piped_child: creation of sockets failed\n");

  he = gethostbyname (command[1]);
  if (!he)
     fatal ("piped_child: unknown host %s\n", command[1]);

  se = getservbyname ("shell", "tcp");
  if (!se)
     fatal ("piped_child: unknown service \"shell\"\n");

  memset (&sa, 0, sizeof(sa));
  sa.sin_addr.s_addr = *(u_long*)he->h_addr;
  sa.sin_port        = ntohs (se->s_port);
  sa.sin_family      = AF_INET;

  if (connect (sock, (struct sockaddr*)&sa, sizeof(sa)) < 0)
     fatal ("piped_child: connection failed: %s\n", strerror(errno));

  p = &buf[0];
  for (i = 2; command[i]; i++)
  {
    strcpy (p, command[i]);
    p += strlen (command[i]);
    *p++ = ' ';
  }
  *(p-1) = '\r';

  if (write_s (sock, buf, (int)(p-buf)) < 0)
     fatal ("piped_child: write failed: %s\n", strerror(errno));

  *f_in  = sock;
  *f_out = sock;
  return (0);
}

int local_child (int argc, char **argv, int *f_in, int *f_out)
{
  int i;

  printf ("local_child():\n");
  for (i = 0; i < argc; i++)
      printf ("  argv[%d] = `%s'\n", i, argv[i]);
  *f_in  = STDIN_FILENO;
  *f_out = STDOUT_FILENO;
  return (0);
}

pid_t do_fork(void)
{
  return (0);
}

void kill_all(int sig)
{
}

int chroot (char *what)
{
  (void)what;   /* to-do !! */
  return (-1);
}

int do_lstat (const char *fname, STRUCT_STAT *st)
{
  (void)fname;
  (void)st;
  return (-1);
}

int do_chmod (const char *path, mode_t mode)
{
  (void)path;
  (void)mode;
  return (-1);
}
#endif
